      subroutine cig2(no,ne,ist,nhci,n,nsg)
c
c    Generates for No space orbitals and Ne electrons
c    all single-excited states + higher excitations
c    in a limited active space between orbitals N1..N2.
c
c     nhci0 .. number of CIs for state generation
c     nhci .. number of generated spin (singlet/triplet) states
c     n number of Slater determinats
c     nhcir .. number of singlets or triplets after selection
c     nnci  ... list of Slater determinats
c     nnci string: nexc k1 j1 k2 j2 .... knexc jnexc, in SO
c     ns1 ... number of singlets
c     ns3 ... number of triplets
c     cic   ... coefficients for spin eigenstates
c     |i>=cic(i,j) nnci(j), i=1..nsi, nsi=ns1 or ns3
c     ist ... requesting singlets: ist=1
c             requesting triplets: ist=3
c     nii00 ... maximum number of excitation
c
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      parameter (nhci0=140000,nort=700,nh110=2*nort,nhcir=100000,
     1nh0=100,nii00=8,nii33=1+2*nii00)
      integer*4 nncig,nnci
      integer*4 nnca(nhci0,nii33)
      common/icicom/nncig(nh110),nnci(nhci0,nii33),
     1icie(nhcir),icid(nhcir),icis(nhcir),
     2isis(nhci0),isie(nhci0)
      common/rcicom/cic(nhcir,nh0)
      common/opts/lopt(300),icha,n11
      common/iopts/iopt(300)
      logical lopt,lsprint,lciss
      equivalence (lsprint,lopt(1)),(iopt(16),neci),
     1(iopt(8),n1),(iopt(9),n2),(lopt(71),lciss)
      logical londisk
c
      write(3,*)'starting cig2',no
      write(6,*)'starting cig2'
      ns=2*no
      if(ns.gt.nh110)call report('Too many orbitals')
      if(ne.gt.ns)call report('Too many electrons')
      if(n1.lt.0)call report('n1<0')
      if(n2.gt.no)call report('n2>no')
      noa=n2-n1+1
      nsa=2*noa
      nea=ne-2*(n1-1)
c
      if(ist.eq.1)inquire(file='CI.SCR',exist=londisk)
      if(ist.eq.3)inquire(file='CIT.SCR',exist=londisk)
      if(ist.ne.1.and.ist.ne.3)call report('unknown multiplicity')
c
      if(londisk)THEN
c
      if(ist.eq.1)open(33,file='CI.SCR',form='unformatted')
      if(ist.eq.3)open(33,file='CIT.SCR',form='unformatted')
      read(33)ns,n,(nncig(ii),ii=1,ns)
      read(33)((nnci(i,j),i=1,n),j=1,nii33)
      read(33)nhci,(icid(i),i=1,nhci),(icis(i),i=1,nhci),
     1(icie(i),i=1,nhci)
      read(33)nsg,(isis(i),i=1,nsg),(isie(i),i=1,nsg)
      read(33)((cic(is,ii-icis(is)+1),ii=icis(is),icie(is)),is=1,nhci)
      close(33)
      if(ist.eq.1)call w36('ci states read from CI.SCR')
      if(ist.eq.3)call w36('ci states read from CIT.SCR')
c
      ELSE
c
c     generate single excitations:
      nx=1
      if(ist.eq.1)open(33,file='CI.STA')
      if(ist.eq.3)open(33,file='CIT.STA')
c     if lciss then all CIS, or only for the active space
      if(lciss)then
       ne1=ne
       no1=no
      else
       ne1=nea
       no1=noa
      endif
      call cige2(ne1,no1,nx,n,nnci,nncig)
      write(3,6001)ne1,2*no1,n1,n2,n
      write(33,6001)ne1,2*no1,n1,n2,n
6001  format(i4,' electrons distributed over ',I4,' spinorbitals',/,
     1'active space : ',i4,' - ',i4,/,
     2'number of nonoexcited states + ground state:',i6)
      call writest(nnci,nhci0,n,0,nii33)
      call getmz0(nnci,nhci0,n,ne1,nii33)
      if(.not.lciss)call addpassive(nnci,nhci0,n,n1,nii33)
      call writest(nnci,nhci0,n,1,nii33)
      write(33,*)'single exc have been generated as above'
      write(6,*)'single exc have been generated '
c
c     generate active space excitations:
      write(6,888)nsa,nea,fact(nsa)/fact(nsa-nea)/fact(nea)
      write(3,888)nsa,nea,fact(nsa)/fact(nsa-nea)/fact(nea)
888   format(i6,' s-orbitals and ',i4,' electrons in the active space',
     1/,'ns!/(ns-ne)!/ne! = ',F50.0)
      write(6,*)'Maximum excitation (-2:Full CI):',neci
      call cige2(nea,noa,neci,na,nnca,nncig)
      call writest(nnca,nhci0,na,-1,nii33)
      call writest(nnca,nhci0,na,0,nii33)
      write(6,*)'written'
      call getmz0(nnca,nhci0,na,nea,nii33)
      call writest(nnca,nhci0,na,0,nii33)
      write(6,*)'mz0'
      call getmae(nnca,nhci0,na,neci,nii33)
      write(6,*)'mae1'
      call writest(nnca,nhci0,na,0,nii33)
      call addpassive(nnca,nhci0,na,n1,nii33)
      write(6,*)'mae2'
      call writest(nnca,nhci0,na,0,nii33)
      write(33,*)'active space excitations above'
      write(3,*)na,' states in active space remained'
      write(6,*)na,' states in active space remained'
c
c     add to monoexcited and get rid of redundant states:
      ndel=0
      do 1 ia=1,na
      nexcia=nnca(ia,1)
      do 2 i=1,n
      nexci=nnci(i,1)
      if(nexci.ne.nexcia)goto 2
      do 5 ii=1,nexci+nexci
5     if(nnca(ia,1+ii).ne.nnci(i,1+ii))goto 2
      goto 1
2     continue
c
c     state ia not found, add it to monoexcited:
      ndel=ndel+1
      do 6 ii=1,nexcia+nexcia+1
6     nnci(n+ndel,ii)=nnca(ia,ii)
1     continue
      n=n+ndel
c
      call writest(nnci,nhci0,n,1,nii33)
      write(3,*)n,' different states, total as above'
      write(33,*)n,' different states, total as above'
c
c     make ground state config
      ns=no+no
      do 101 ii=1,ns
      if((ii/2)*2.eq.ii)then
       is= 1
      else
       is=-1
      endif
      nncig(ii)=0
101   if(ii.le.ne)nncig(ii)=int(is)
c
c     order orbitals
      write(6,*)'sorting states'
      call sort2(n,nnci,ns,nncig,ne)
      write(3,*)'states sorted, full orbitals first:'
      write(33,*)'states sorted, full orbitals first:'
      call writest(nnci,nhci0,n,0,nii33)
c
      write(6,*)'grouping states'
      call order(n,nnci)
      write(33,*)'after order:'
      call writest(nnci,nhci0,n,1,nii33)
      call sgroups(nnci,n,no,nsg,ns1,ns3,ist,cic,icis,icie,icid,
     1isis,isie,nii33,nncig)
      if(ist.eq.1)then
       write(33,*)ns1,' singlets selected'
       write(6,*)ns1,' singlets selected'
       nhci=ns1
      else
       write(33,*)ns3,' triplets selected'
       write(6,*)ns3,' triplets selected'
       nhci=ns3
      endif
      do is=1,nhci
       write(33,3000)is,icid(is),icis(is),icie(is)
3000   format(i5,' dim = ',i7,'; from det ',i7,' to ',i7)
       if(lsprint)then
        do ii=icis(is),icie(is)
         iii=ii-icis(is)+1
         write(33,3001)cic(is,iii)
3001     format(f10.5,' of ')
         write(33,3002)(nnci(ii,jj),jj=1,1+nnci(ii,1)*2)
3002     format(i3,20(i3,'->',i3))
        enddo
       endif
      enddo
      close(33)
      if(ist.eq.1)open(33,file='CI.SCR',form='unformatted')
      if(ist.eq.3)open(33,file='CIT.SCR',form='unformatted')
      write(33)ns,n,(nncig(ii),ii=1,ns)
      write(33)((nnci(i,j),i=1,n),j=1,nii33)
      write(33)nhci,(icid(i),i=1,nhci),(icis(i),i=1,nhci),
     1(icie(i),i=1,nhci)
      write(33)nsg,(isis(i),i=1,nsg),(isie(i),i=1,nsg)
      write(33)((cic(is,ii-icis(is)+1),ii=icis(is),icie(is)),is=1,nhci)
      close(33)
      if(ist.eq.1)call w36('ci states written to CI.STA and CI.SCR')
      if(ist.eq.3)call w36('ci states written to CIT.STA and CIT.SCR')
      ENDIF
c
      return
      end
c     ==================================================================
      subroutine cige2(ne,no,nx,n,nnci,nncig)
c
c     generates CI states, for closed shells so far
c     ne    number of electrons
c     no    number of space orbitals
c     nx    how many times excited
c     nhci0 maximum dimension of CI Hamiltonian
c     nh110 max number of spinorbitals
c
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      parameter (nhci0=140000,nort=700,nh110=2*nort,
     1nii00=8,nii33=1+2*nii00,nhci0fci=8750000,nh110ci=60)
      integer*4 nncig(nh110),nncitem(nhci0fci,nh110ci),
     1newc(nh110ci)
      dimension nnci(nhci0,nii33)

      write(6,600)ne,no,nx
600   format(' cige2 ',/,
     1       '    ne ',i5,/,
     1       '    no ',i5,/,
     1       '    nx ',i5,/)
c
      ns=2*no
      do 101 ii=1,ns
      if((ii/2)*2.eq.ii)then
       is= 1
      else
       is=-1
      endif
      nncig(ii)=0
101   if(ii.le.ne)nncig(ii)=is
c
      if(nx.eq.-2.or.nx.gt.3)then
        if(ns.gt.nh110ci)write(6,*)ns,nh110ci
        if(ns.gt.nh110ci)call report('nh110ci<ns')
c       distribute electrons over spinorbitals
c
c       distribute first electron, ns possibilities:
        n=0
        do 52 i=1,ns
        n=n+1
        do 104 ii=1,ns
104     nncitem(n,ii)=0
52      nncitem(n,i)=1
c
c       distribute rest of electrons:
        do 51 ie=2,ne
        ndel=0
c
c       look at each state nn:
        do 56 nn=1,n
c
c       find where was last electron
        do 53 is=ns,1,-1
53      if(nncitem(nn,is).eq.ie-1)goto 500
        write(6,*)'ns=',ns
        write(6,*)'n=',n
        write(6,*)'ne=',ne
        write(6,*)'nn=',nn
        write(6,*)'nncitem(nn)=',(nncitem(ii,is),is=1,ns)
        call report('error in code')
c
c       put the electron from last to right hand side
500     do 54 ii=is+1,ns
c
c       place electron ie on place ii
        ndel=ndel+1
        if(n+ndel.gt.nhci0fci)then
         write(3,843)ie,is,ns,ne,nx,n,ndel,nhci0fci
         write(6,843)ie,is,ns,ne,nx,n,ndel,nhci0fci
843      format('ie        = ',i6,/
     1          'is        = ',i6,/
     2          'ns        = ',i6,/
     2          'ne        = ',i6,/
     2          'nx        = ',i6,/
     3          'n         = ',i16,/
     4          'ndel      = ',i16,/
     4          'nhci0fci  = ',i16)
         do i=1,n+ndel-1
          write(3,8432)i,(nncitem(i,ij),ij=1,ns)
8432      format(i10,70i1)
         enddo
         call report('too many config cige2')
        endif
c
        do 55 ij=1,ns
55      newc(ij)=nncitem(nn,ij)
        newc(ii)=ie
        do 551 ij=1,ns
551     nncitem(n+ndel,ij)=newc(ij)
c
c       exclude high spin states and high excited states
        ispi=0
        iexc=0
        do 560 i=1,ns
        if(i/2*2.ne.i)then
         it=-1
        else
         it=1
        endif
        if(newc(i).ne.0.and.i.gt.ne)iexc=iexc+1
560     if(newc(i).ne.0)ispi=ispi+it
        if((nx.gt.0.and.iexc.gt.nx).
     1  or.abs(ispi).gt.(ne+1)/2.
     2  or.(ii.eq.ns.and.ie.lt.ne) )ndel=ndel-1
c
c
54      continue
c
c       mark state that could not be used:
        if(is.eq.ns)nncitem(nn,1)=-9
c
56      continue
c       next state

c
        n=n+ndel
c
c       get rid of states that could not be used:
        nt=1
9002    ic=0
        do 420 nn=nt,n
        if(nncitem(nn,1).eq.-9)then
         do 430 ii=1,ns
430      nncitem(nn,ii)=nncitem(n,ii)
         ic=1
         goto 9001
        endif
420     continue
9001    if(ic.eq.1)then
         nt=nn
         n=n-1
         goto 9002
        endif
c
c       get rid of initial states
        ii=0
2000    ii=ii+1
        if(ii.gt.n)goto 2001
        nel=0
        do 62 jj=1,ns
62      if(nncitem(ii,jj).gt.0)nel=nel+1
        if(nel.ne.ie)then
         do 63 jj=1,ns
63       nncitem(ii,jj)=nncitem(n,jj)
         ii=ii-1
         n=n-1
        endif
        goto 2000
2001    continue
c
c        do 561 iu=1,n
c561     write(6,6900)iu,(nncitem(iu,iuu),iuu=1,ns)
c6900    format(i10,20i2)
c        write(6,*)ie,ne
c        read(5,*)ioo

c       (distribute next electron:)
51      continue
c
c       set spins
        do 81 is=1,n
        do 81 ii=1,ns
        if(2*(ii/2).eq.ii)then
         ispin= 1
        else
         ispin=-1
        endif
81      if(nncitem(is,ii).ne.0)nncitem(is,ii)=ispin
c
c       rewrite to nnci
        do 82 is=1,n
        nexc1=0
        do 83 ii=1,ne
        if(nncitem(is,ii).ne.nncig(ii))then
         nexc1=nexc1+1
         if(1+(2*nexc1-1).gt.nii33)call report('nexc1 overflows nii3')
         nnci(is,1+(2*nexc1-1))=ii
        endif
83      continue
        nexc2=0
        do 84 ii=ne+1,ns
        if(nncitem(is,ii).ne.nncig(ii))then
         nexc2=nexc2+1
         if(1+(2*nexc2-1).gt.nii33)call report('nexc2 overflows nii3')
         nnci(is,1+(2*nexc2  ))=ii
        endif
84      continue
        if(nexc1.ne.nexc2)then
         write(6,*)(nncig(ii),ii=1,ns)
         write(6,*)(nncitem(is,ii),ii=1,ns)
         write(6,*)'ns:',ns
         write(6,*)'ne:',ne
         write(6,*)nexc1,nexc2
         call report('nexc1<>nexc2 cige2')
        endif
82      nnci(is,1)=nexc1
c
      else
c
c       (excitations up to double-)
        ndo=ne/2
        nvir=no-ndo
c
c       ground state:
        n=1
        nnci(1,1)=0
        write(6,*)n,' ground state'
c
        if(nx.lt.1)return
c
c       monoexcited, don't change spin:
        write(6,*)ndo*nvir*2,' mono-excited states'
        do 7 k=1,ndo
        do 7 iss=1,2
        do 7 j=ndo+1,no
        n=n+1
        if(n.gt.nhci0)call report('too many config')
        nnci(n,1)=1
        nnci(n,2)=2*k-2+iss
7       nnci(n,3)=2*j-2+iss
c
        if(nx.lt.2)return
c
c       double excited
        npred=ndo*nvir*(2*ndo*nvir+(ndo-1)*(nvir-1))/2
        nfound=0
        do 10 k=1,ndo
        do 10 iss=1,2
        isk =k +k -2+iss
        do 10 kp=k,ndo
        do 10 issp=1,2
        iskp=kp+kp-2+issp
        if(iskp.le.isk)goto 10
        do 12 j =ndo+1,no
        isj =j +j -2+iss
        do 12 jp=ndo+1,no
        isjp=jp+jp-2+issp
        if(isjp.eq.isj)goto 12
        if(iss.eq.issp.and.isjp.lt.isj)goto 12
        n=n+1
        nfound=nfound+1
        if(n.le.nhci0)then
         nnci(n,1)=2
         nnci(n,2)=isk
         nnci(n,3)=isj
         nnci(n,4)=iskp
         nnci(n,5)=isjp
        endif
12      continue
10      continue
        write(6,*)nfound,' double-excited states'
        if(nfound.ne.npred)call report('nfound <> npred')
        if(n.gt.nhci0)write(6,*)n,nhci0
        if(n.gt.nhci0)call report('too many config')

        if(nx.lt.2)return
c
c       triple excited
        nfound=0
        do 13 k=1,ndo
        do 13 iss=1,2
        isk =k +k -2+iss
        do 13 kp=k,ndo
        do 13 issp=1,2
        iskp=kp+kp-2+issp
        if(iskp.gt.isk)then
         do 131 k3=kp,ndo
         do 131 iss3=1,2
         isk3=k3+k3-2+iss3
         if(isk3.gt.iskp)then
          do 14 j =ndo+1,no
          isj =j +j -2+iss
          do 14 jp=ndo+1,no
          isjp=jp+jp-2+issp
          if(isjp.eq.isj)goto 14
          if(iss.eq.issp.and.isjp.lt.isj)goto 14
          do 141 j3=ndo+1,no
          isj3=j3+j3-2+iss3
          if(isj3.eq.isj)goto 141
          if(isj3.eq.isjp)goto 141
          if(iss.eq.issp.and.isjp.lt.isj)goto 141
          if(iss.eq.iss3.and.isj3.lt.isj)goto 141
          if(issp.eq.iss3.and.isj3.lt.isjp)goto 141
          n=n+1
          nfound=nfound+1
          if(n.le.nhci0)then
           nnci(n,1)=3
           nnci(n,2)=isk
           nnci(n,3)=isj
           nnci(n,4)=iskp
           nnci(n,5)=isjp
           nnci(n,6)=isk3
           nnci(n,7)=isj3
          endif
141       continue
14        continue
         endif
131      continue
        endif
13      continue
        write(6,*)nfound,' triple-excited states'
        if(n.gt.nhci0)write(6,*)n,nhci0
        if(n.gt.nhci0)call report('too many config')

      endif
c
      return
      end
c     ==========================================================
      subroutine order(n,nnci)
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      parameter (nhci0=140000,
     1nii00=8,nii33=1+2*nii00)
      dimension nnci(nhci0,nii33)
c
c     order SOs
1     ic=0
      do 2 nn=1,n
      if(nnci(nn,1).gt.1)then
       do 3 i=1,nnci(nn,1)-1
       k =nnci(nn,1+2* i   -1)
       kp=nnci(nn,1+2*(i+1)-1)
       j =nnci(nn,1+2* i     )
       jp=nnci(nn,1+2*(i+1)  )
       if(k.gt.kp)then
        nnci(nn,1+2* i   -1)=kp
        nnci(nn,1+2*(i+1)-1)=k
        ic=1
       endif
       if(j.gt.jp)then
        nnci(nn,1+2* i     )=jp
        nnci(nn,1+2*(i+1)  )=j
        ic=1
       endif
3      continue
      endif
2     continue
      if(ic.gt.0)goto 1
c
c     write(33,*)'after order SO:'
c     call writest(nnci,nhci0,n,1,nii33)
c
c     group same space config
      nstart=1
888   do 30 nn=nstart,n
      ne1=nnci(nn,1)
      do 30 np=nn+1,n
      nep=nnci(np,1)
      if(ne1.ne.nep)goto 30
      do 32 ii=1,ne1
      k1=(nnci(nn,1+2*ii-1)+1)/2
      j1=(nnci(nn,1+2*ii  )+1)/2
      kp=(nnci(np,1+2*ii-1)+1)/2
      jp=(nnci(np,1+2*ii  )+1)/2
      if(k1.ne.kp)goto 7777
32    if(j1.ne.jp)goto 7777
c
c     np has same space configuration as nn, move np behind nn and
c     start over
      mb=1+2*max(nnci(nn+1,1),nnci(np,1))
      do 43 ii=1,mb
43    call swi(nnci(nn+1,ii),nnci(np,ii))
      nstart=nn+1
      goto 888
7777  continue
30    continue
c
      write(6,*)'states have been ordered - group same space config'
      write(3,*)'states have been ordered - group same space config:'
c
      return
      end
c     ==========================================================
      function fact(n)
      integer*4 n,i
      real*8 fact,ff,as
      as=0.0d0
      ff=1.0d0
      do 1 i=1,n
      as=as+1.0d0
1     ff=ff*as
      fact=ff
      return
      end
c     =====================
      subroutine writest(nnci,nhci0,n,ic,nii33)
      integer*4 n,ic,i,j,nhci0,nii33,ie,ifrom,is,ito
      integer*4 nnci(nhci0,nii33)
c
      write(33,6000)n
6000  format(I6,' states:')
c
      if(ic.ne.-1)then
c      check that spin is zero
       do 11 i=1,n
       is=0
       do 12 ie=1,nnci(i,1)
       ifrom=nnci(i,1+ie+ie-1)
       ito  =nnci(i,1+ie+ie)
       if((ito/2)*2.eq.ito)then
        is=is+1
       else
        is=is-1
       endif
       if((ifrom/2)*2.eq.ifrom)then
        is=is-1
       else
        is=is+1
       endif
12     continue
       if(is.ne.0)then
         write(33,6001)i,(nnci(i,j),j=1,2*nnci(i,1)+1)
         write(6,6001)i,(nnci(i,j),j=1,2*nnci(i,1)+1)
         call report('change of spin writest')
       endif
11     continue
      endif
c
      if(ic.le.0)return
c
      do 1 i=1,n
1     write(33,6001)i,(nnci(i,j),j=1,2*nnci(i,1)+1)
6001  format(i8,i3,20(i3,'->',i3))
      return
      end
c     =====================
      subroutine addpassive(nnci,nhci0,n,n1,nii33)
c
c     adds passive orbitals, n1 ... start of space orb window
c     passive    1..................2*nmo SO
c     active          n1...n1+no*2-1      AO
c                1..np                    SO
c
      integer*4 nhci0,nii33,n,i,j,nnci(nhci0,nii33),np,n1
c
      np=2*(n1-1)
      do 1 i=1,n
      do 1 j=1,nnci(i,1)*2
1     nnci(i,1+j)=nnci(i,1+j)+np
      return
      end
c     =====================
      subroutine getmz0(nnci,nhci0,n,ne,nii33)
c
c     select states only with |Mz| same as ground
      integer*4 nhci0,n,i,j,mz,ii,jj,mzl,ne,nii33
      integer*4 nnci(nhci0,nii33),istart,k,ks,l,ls
c
      if(2*(ne/2).eq.ne)then
       mzl=0
      else
       mzl=-1
      endif
c
      istart=1
4     do 1 i=istart,n
      mz=mzl
      do 2 j=1,nnci(i,1)
      k=nnci(i,1+2*j-1)
      l=nnci(i,1+2*j  )
      ks=-1
      if((k/2)*2.eq.k)ks=1
      ls=-1
      if((l/2)*2.eq.l)ls=1
2     mz=mz-ks+ls
c
c     MZ<>mzl, get rid of this state:
      if(mz.ne.mzl)then
       do 5 ii=i,n-1
       do 5 jj=1,2*nnci(ii+1,1)+1
5      nnci(ii,jj)=nnci(ii+1,jj)
       istart=i
       n=n-1
       goto 4
      endif
1     continue
      write(3,*)'only states retained with Mz=',mzl
      write(6,*)'only states retained with Mz=',mzl
      return
      end
c     ===========================================================
      subroutine getmae(nnci,nhci0,n,nexc,nii33)
c
c     get only states excited less than nexc times
c
      integer*4 nhci0,n,i,ii,jj,nexc,nii33
      integer*4 istart
      integer*4 nnci(nhci0,nii33)
c
      if(nexc.lt.0)then
       write(3,*)'excitation not limited'
       return
      endif
c
      istart=1
4     do 1 i=istart,n
c
c     MZ>nexc, get rid of this state:
      if(nnci(i,1).gt.nexc)then
       do 5 ii=i,n-1
       do 5 jj=1,2*nnci(ii+1,1)+1
5      nnci(ii,jj)=nnci(ii+1,jj)
       istart=i
       n=n-1
       goto 4
      endif
1     continue
      write(6,*)'states retained with exc limit',nexc
      write(3,*)'states retained with exc limit',nexc
      return
      end
c     =============================================================
      subroutine tcin(nnci,nncig,ni,nh110,is,ns)
c     rewrites Slater determinant is into long formate
      integer*4 nh110,nhci0,nii00,nii33,ns
      parameter (nhci0=140000,nii00=8,nii33=1+2*nii00)
      integer*4 nnci(nhci0,nii33),is,i,ifrom,ito
      integer*4 nncig(nh110),ni(nh110)
      do 1 i=1,ns
1     ni(i)=nncig(i)
      do 2 i=1,nnci(is,1)
      ifrom=nnci(is,1+i+i-1)
      ito  =nnci(is,1+i+i)
      ni(ifrom)=0
      if((ito/2)*2.eq.ito)then
       ni(ito)=1
      else
       ni(ito)=-1
      endif
2     continue
      return
      end
c     =============================================================
      subroutine tcini(nnci,nncig,ni,nh110,is,ns,ne)
c     rewrites long formate of Slater determinant is to short
      integer*4 nh110,nhci0,nii00,nii33,ns,ne,ii,nexc1
      parameter (nhci0=140000,nii00=8,nii33=1+2*nii00)
      integer*4 nnci(nhci0,nii33),is,nexc2
      integer*4 nncig(nh110),ni(nh110)
c
      nexc1=0
      do 83 ii=1,ne
      if(ni(ii).ne.nncig(ii))then
       nexc1=nexc1+1
       nnci(is,1+2*nexc1-1)=ii
      endif
83    continue
      nexc2=0
      do 84 ii=ne+1,ns
      if(ni(ii).ne.nncig(ii))then
       nexc2=nexc2+1
       nnci(is,1+2*nexc2  )=ii
      endif
84    continue
      if(nexc1.ne.nexc2)then
       write(6,*)nexc1,nexc2
       write(6,*)(ni(ii),ii=1,ns)
       write(6,*)(nncig(ii),ii=1,ns)
       write(6,*)'ne=',ne
       write(6,*)'ns=',ns
       call report('nexc1<>nexc2')
      endif
      nnci(is,1)=nexc1
      return
      end
c     =============================================================
      subroutine sgroups(nnci,n,no,nsg,ns1,ns3,ist,cic,icis,
     1icie,icid,isis,isie,nii33,nncig)
c
c     get spin groups and eigenfunctions:
c
c     get singlets or triplets
c     ns1 ... number of singlets
c     ns3 ... number of triplets
c     cic ... coefficients  |i>=cic(i,j) nnci(j), i=1..nsi, nsi=ns1 or ns3
c     ist ... requesting singlets: ist=1
c             requesting triplets: ist=3
c
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      parameter (nhci0=140000,nort=700,nh110=2*nort,nh0=100,
     1nhcir=100000)
      integer*4 nnci(nhci0,nii33)
      integer*4 nncig(nh110),ni(nh110)
      dimension h(nh0,nh0),sc(nh0,nh0),se(nh0),cic(nhcir,nh0),
     1icis(nhcir),icie(nhcir),icid(nhcir),isis(nhci0),isie(nhci0)
      character*14 sqs
      dimension et(nh0)
c
      write(6,*)'distinquishing spin groups'
      ifile=33
      ns1=0
      ns3=0
      nso=0
      ns=no+no
      nsg=0
      nn=1
888   nsg=nsg+1
      isis(nsg)=nn
      do 32 np=nn+1,n
      do 32 ii=1,no
c     compare how many electrons are in the space orbital ii
      if(nnci(nn,1).ne.nnci(np,1))goto 7777
      do jj=1,nnci(nn,1)
       kn=nnci(nn,1+2*jj-1)
       jn=nnci(nn,1+2*jj  )
       kp=nnci(np,1+2*jj-1)
       jp=nnci(np,1+2*jj  )
       if((kn+1)/2.ne.(kp+1)/2)goto 7777
       if((jn+1)/2.ne.(jp+1)/2)goto 7777
      enddo
32    continue
      isie(nsg)=n
7777  isie(nsg)=np-1
      if(np.le.n)then
       nn=np
       goto 888
      endif
      write(6,*)nsg,' spin groups'
      write(ifile,*)nsg,' spin groups'
c
      do 1 i=1,nsg
      i1=isis(i)
      i2=isie(i)
      write(ifile,2000)i,i1,i2
2000  format(/,'Group ',i6,' from ',i6,' to ',i6,':')
c
c     diagonalize S^2
      nhdim=i2-i1+1
      if(nhdim.gt.nh0)call report('nhdim > nh0')
c
c     isn  ... number of singly occupied orbitals:
      isn=0
      call tcin(nnci,nncig,ni,nh110,i1,ns)
      do 90 i8=1,ns-1,2
90    if(ni(i8).ne.0.and.ni(i8+1).eq.0.or.
     1   ni(i8).eq.0.and.ni(i8+1).ne.0)isn=isn+1
c
      do 2 ii=i1,i2
      ii1=ii-i1+1
      do 2 jj=i1,i2
      jj1=jj-i1+1
2     h(ii1,jj1)=ssquared(ii,jj,nnci,ns,nncig)
      write(ifile,3001)nhdim,nhdim
3001  format('Spin Hamiltonian (',I2,' x ',I2,'):')
      do 4 ii=1,nhdim
4     write(ifile,3000)(h(ii,jj),jj=1,nhdim)
3000  format(10f8.3)
      IMOD=2
      CALL TRED12(nhdim,h,sc,se,IMOD,IERR,nh0,et)
      if(IERR.ne.0)call report('diagonalization error')
      write(ifile,*)'Eigenvalues:'
      do 3 ie=1,nhdim
      sq=(-1.0d0+sqrt(1.0d0+4.0d0*se(ie)))*0.5d0
      if(    sq       .lt.-0.01d0)call report('negative spin')
      itake=0
      if(    sq       .lt. 0.01d0)then
       sqs='    singlet   '
       ns1=ns1+1
       n11=ns1
       if(ist.eq.1)itake=1
      else
       if(abs(sq-1.0d0).lt. 0.01d0)then
        sqs='    triplet   '
        ns3=ns3+1
        n11=ns3
        if(ist.eq.3)itake=1
       else
        nso=nso+1
        n11=nso
        sqs=' general spin '
       endif
      endif
      write(ifile,3002)ie,se(ie),sq,sqs,n11
      ifn=nint(fact(isn))
      iq=isn/2-nint(sq)
      write(ifile,3022)ifn/fact(iq)/fact(isn-iq)
     1                   -ifn/fact(iq-1)/fact(isn-iq+1)
3022  format('(n!/(n-q)!/(q!)-n!/(n-q+1)!/(q-1)! = ',f10.0)
3002  format(I2,'. eigenvalue: S(S+1)=',f7.4,' S = ',f7.4,a14,i7)
      write(ifile,*)'Eigenvector'
      write(ifile,3003)(sc(jj,ie),jj=1,nhdim)
3003  format(10f8.4)
      if(itake.eq.1)then
       if(n11.gt.nhcir)write(6,*)n11,nhcir
       if(n11.gt.nhcir)call report('Too many spin states')
       icis(n11)=i1
       icie(n11)=i2
       icid(n11)=i2-i1+1
       do 5 jj=1,nhdim
5      cic(n11,jj)=sc(jj,ie)
      endif
3     continue
      write(ifile,*)'Determinants:'
      do 1 j=i1,i2
1     write(ifile,6001)(nnci(j,k),k=1,2*nnci(j,1)+1)
6001  format(i3,20(i3,'->',i3))
c
      write(ifile,*)ns1,' singlets'
      write(6,*)ns1,' singlets'
      write(ifile,*)ns3,' triplets'
      write(6,*)ns3,' triplets'
      write(ifile,*)nso,' others'
      write(6,*)nso,' others'
      write(6,*)ns3+ns1+nso,' total'
      write(ifile,*)ns3+ns1+nso,' total'
      return
      end
c     =============================================================
      function ssquared(i,j,nnci,ns,nncig)
c
c     <i|S^2|j>
c     make use of the generalized Dirac identity
c     S^2F={sum(ab)Pab +(1/4)[(na-nb)**2+2na+2nb]}F
c     Pab ... all possible interchanges of a and b spins in the F determinant
c
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      parameter (nhci0=140000,nort=700,nh110=2*nort,
     1nii00=8,nii33=1+2*nii00)
      integer*4 nncig(nh110),ni(nh110),nj(nh110),nt(nh110),
     1nza(nh110),nzb(nh110),i2,j2
      integer*4 nnci(nhci0,nii33)
      logical lsame
c
      hij=0.0d0
      j9=j
      i9=i
      call tcin(nnci,nncig,nj,nh110,j9,ns)
      call tcin(nnci,nncig,ni,nh110,i9,ns)
      do 1 ii=1,ns
1     nt(ii)=nj(ii)
c
c     number of alpha and beta electrons in |j>:
      na=0
      nb=0
      do 2 ii=1,ns
c
c     skip double occupied orbitals (which have S=Sz=0):
      io1=((ii-1)/2)*2+1
      io2=io1+1
      if(nj(io1).ne.0.and.nj(io2).ne.0)goto 2
c
      if(nj(io1).ne.0.and.ii.eq.io1)then
       na=na+1
       nza(na)=io1
      endif
      if(nj(io2).ne.0.and.ii.eq.io2)then
       nb=nb+1
       nzb(nb)=io2
      endif
2     continue
      nr=na+nb
      nd=na-nb
c
      if(lsame(ni,nj,ns).and.nr.gt.0)hij=hij+0.25d0*dble(nd**2+nr+nr)
c
c     take all possible self standing (mono occ) electrons alpha:
c     interchange them with similar all electrons beta:
      do 3 ia=1,na
      i2=nza(ia)
      io1=((i2-1)/2)*2+1
      io2=io1+1
      do 3 ib=1,nb
      j2=nzb(ib)
      jo1=((j2-1)/2)*2+1
      jo2=jo1+1
c
c     put alpha into same space orbital as beta
      nt(jo1)=nt(io1)
c
c     delete alpha in original orbital
      nt(io1)=0
c
c     move beta analogously:
      nt(io2)=nt(jo2)
      nt(jo2)=0
c
      if(lsame(ni,nt,ns))hij=hij+1.0d0
c
c     move it back:
      nt(io1)=nt(jo1)
      nt(jo1)=0
      nt(jo2)=nt(io2)
      nt(io2)=0
3     continue
c
      ssquared=hij
c
      return
      end
c     =============================================================
      function lsame(ni,nj,ns)
      integer*4 ns,i
      integer*4 ni(ns),nj(ns)
      logical lsame
c
      lsame=.false.
      do 1 i=1,ns
1     if(ni(i).ne.nj(i))return
      lsame=.true.
      return
      end
c     =============================================================
      function lbigger(a,j,at,ns,nncig)
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      parameter (nhci0=140000,nort=700,nh110=nort+nort,
     1nii00=8,nii33=1+2*nii00)
      integer*4 a(nhci0,nii33)
      integer*4 tt(nh110),at(nh110),nncig(nh110)
      logical lbigger
c
      lbigger=.false.
      call tcin(a,nncig,tt,nh110,j,ns)
      do 1 ii=1,ns
1     if(tt(ii).ne.0.or.at(ii).ne.0)goto 2
2     if(tt(ii).ne.0.and.at(ii).eq.0)lbigger=.true.
      return
      end
C     ===================================================
      function lsmaller(a,j,at,ns,nncig)
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      parameter (nhci0=140000,nort=700,nh110=nort+nort,
     1nii00=8,nii33=1+2*nii00)
      integer*4 a(nhci0,nii33)
      integer*4 at(nh110),tt(nh110),nncig(nh110)
      logical lsmaller
c
      lsmaller=.false.
      call tcin(a,nncig,tt,nh110,j,ns)
      do 1 ii=1,ns
1     if(tt(ii).ne.0.or.at(ii).ne.0)goto 2
2     if(tt(ii).eq.0.and.at(ii).ne.0)lsmaller=.true.
      return
      end
C     ===================================================
      function lb2(a,j,i,ns,nncig)
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      parameter (nhci0=140000,nort=700,nh110=2*nort,
     1nii00=8,nii33=1+2*nii00)
      integer*4 a(nhci0,nii33)
      integer*4 tt(nh110),nncig(nh110)
      integer*4 t2(nh110)
      logical lb2
      lb2=.false.
      call tcin(a,nncig,tt,nh110,j,ns)
      call tcin(a,nncig,t2,nh110,i,ns)
      do 1 ii=1,ns
1     if(tt(ii).ne.0.or.t2(ii).ne.0)goto 2
2     if(tt(ii).ne.0.and.t2(ii).eq.0)lb2=.true.
      return
      end
c     ===================================================
      subroutine sort2(n,arr,ns,nncig,ne)
c     Quicksort
c     M ... when buble sort is switched on
c
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      parameter (nhci0=140000,nort=700,nh110=nort*2,
     1nii00=8,nii33=1+2*nii00)
      integer*4 arr(nhci0,nii33)
      integer*4 a(nh110),nncig(nh110)
      integer*4 n,M,nstack
      parameter (M=10,NSTACK=500)
      integer*4 i,ir,j,jstack,k,l,istack(NSTACK)
      logical lbigger,lb2,lsmaller
c
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
c      insertion sort for small Ms:
       do j=l+1,ir
        call tcin(arr,nncig,a,nh110,j,ns)
        do i=j-1,l,-1
c        if(arr(i).le.a)goto 2
         if(.not.lsmaller(arr,i,a,ns,nncig))goto 2
         do ii=1,1+2*arr(i,1)
          arr(i+1,ii)=arr(i,ii)
         enddo
        enddo
        i=l-1
2       call tcini(arr,nncig,a,nh110,i+1,ns,ne)
       enddo
       if(jstack.eq.0)return
c      pop stack and begind new round of partitioning
       ir=istack(jstack)
       l=istack(jstack-1)
       jstack=jstack-2
      else
c      choose median of left, center and right elements as partitioning
c      element a. Also rearrange so that a(l)<=a(l+1)<=a(ir)
       k=(l+ir)/2
       iim=1+2*max(arr(k,1),arr(l+1,1))
       do ii=1,iim
        call swi(arr(l+1,ii),arr(k,ii))
       enddo
       if(lb2(arr,ir,l,ns,nncig))then
        do ii=1,1+2*max(arr(l,1),arr(ir,1))
         call swi(arr(l,ii),arr(ir,ii))
        enddo
       endif
       if(lb2(arr,ir,l+1,ns,nncig))then
        do ii=1,1+2*max(arr(l+1,1),arr(ir,1))
         call swi(arr(l+1,ii),arr(ir,ii))
        enddo
       endif
       if(lb2(arr,l+1,l,ns,nncig))then
        do ii=1,1+2*max(arr(l+1,1),arr(l,1))
         call swi(arr(l+1,ii),arr(l,ii))
        enddo
       endif
c      initialize pointers for partitioning
       i=l+1
       j=ir
c      partitioning element
       call tcin(arr,nncig,a,nh110,l+1,ns)
c      beginninf of innermost loop
3      continue
c      scan up to find element >a
       i=i+1
       if(lbigger(arr,i,a,ns,nncig))goto 3
c      if(arr(i).lt.a)goto 3
4      continue
c      scan down to find element <a
       j=j-1
       if(lsmaller(arr,j,a,ns,nncig))goto 4
c      if(arr(j).gt.a)goto 4
c      pointers crossed; exit whe partitioning complete
       if(j.lt.i)goto 5
c      exchange elements of both arrays
       do ii=1,1+2*max(arr(i,1),arr(j,1))
        call swi(arr(i,ii),arr(j,ii))
       enddo
       goto 3
c      end of innermost loop
c      insert partitioning elements in both arrays
5      do ii=1,1+2*arr(j,1)
        arr(l+1,ii)=arr(j,ii)
       enddo
       call tcini(arr,nncig,a,nh110,j,ns,ne)
       jstack=jstack+2
c      push pointers to larger subarray on stack,
c      process smaller subarray immediately
       if(jstack.gt.nstack)write(6,*)'nstack too small'
       if(ir-i+1.ge.j-l)then
        istack(jstack)=ir
        istack(jstack-1)=i
        ir=j-1
       else
        istack(jstack)=j-1
        istack(jstack-1)=l
        l=i
       endif
      endif
      goto 1
      end

