      subroutine cigd(no,nhci,n)
c
c     makes CI elements of one-electron moments, writes it to AP.SCR
c     e.g.  <spin adapted state i|el.moment|spin adapted state j>
c     n .. number of Slater determinants
c     nhci .. number of selected spin states (singlets or triplets)
c     nsg ... number of spin groups of determinants
c
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      parameter (nhci0=140000,nort=700,nh110=2*nort,nhcir=100000,
     1nh0=100,nii00=8,nii33=1+2*nii00,tol=1.0d-10,nel=15)
      integer*4 nncig
      common/icicom/nncig(nh110),nnci(nhci0,nii33),
     1icie(nhcir),icid(nhcir),icis(nhcir),
     2isis(nhci0),isie(nhci0)
      common/rcicom/cic(nhcir,nh0)
      integer*4 is(nh110),js(nh110)
      dimension h(nhci0),ind(nhci0),
     1h1(nh0,nh0),h2(nh0),h3(nh0),O(nort,nort)
      character*8 fn(nel)
      data fn/' APX.SCR',' APY.SCR',' APZ.SCR',
     1' AQ1.SCR',' AQ2.SCR',' AQ3.SCR',' AQ4.SCR',' AQ5.SCR',' AQ6.SCR',
     1' ALX.SCR',' ALY.SCR',' ALZ.SCR',
     1' AVX.SCR',' AVY.SCR',' AVZ.SCR'/
      logical londisk
c
      write(3,*)'starting cigd, nhci= ',nhci,' n = ',n
      write(6,*)'starting cigd, nhci= ',nhci,' n = ',n
      if(n.gt.nhci0)call report('n>nhci0 in cigd')
      if(nhci.gt.nhcir)call report('nhci>nhcir in cigd')
      write(6,*)'making integrals in spin-adapted basis'
      ns=2*no
c
      do 10000 io=1,nel
      inquire(file=fn(io),exist=londisk)
      if(londisk)then
       write(6,*)fn(io),' found on disk'
       goto 10000
      endif
c
c     get in memory the integral in MO basis:
      call iin2(io,O,no)
c
c     Transition integrals in spin-adapted functions 1..nhci
      open(20,file=fn(io),form='unformatted')
      write(20)nhci
c
c     number of non-zero elements:
      inz=0
c
c     number of all elements (calculated):
      int=0
      ii=0
c
c     Loop over ii
c     ii,jj .. spin adapted basis function index
c     id,jd .. determinant index
101   ii=ii+1
      if(ii.gt.nhci)goto 999
      istart=icis(ii)
      nhdimi=icid(ii)
c
c     spin states iistart...iiend consist of determinands istart..(iend)
c     find iiend, iistart
      do 5 iiend=ii,nhci
5     if(icis(iiend).ne.istart)goto 6
      iiend=nhci+1
6     iiend=iiend-1
      do 52 iistart=ii,1,-1
52    if(icis(iistart).ne.istart)goto 62
      iistart=0
62    iistart=iistart+1
c
      jj=0
c
c     number of non-zero elements in row ii:
      jt=0
c
c     Loop over jj, record only jj<=ii
102   jj=jj+1
      if(jj.gt.nhci)goto 888
      jstart=icis(jj)
      nhdimj=icid(jj)
c
c     spin states jj...jjend consist of determinands jstart..(jend)
c     find jjend and process all states jj..jjend:
c
      do 51 jjend=jj,nhci
51    if(icis(jjend).ne.jstart)goto 61
      jjend=nhci+1
61    jjend=jjend-1
c
c     get matrix elements with small determinants
      do 1 id=1,nhdimi
      call tcin(nnci,nncig,is,nh110,id+istart-1,ns)
      do 1 jd=1,nhdimj
      call tcin(nnci,nncig,js,nh110,jd+jstart-1,ns)
      call dhsdp(hjj,is,js,ns,O)


      do 1 iee=1,nel
1     h1(id,jd)=hjj
c
c     transform into big - only for ii
c     first index
      do 7 jd=1,nhdimj
      s=0.0d0
      do 8 id=1,nhdimi
8     s=s+cic(ii,id)*h1(id,jd)
7     h2(jd)=s
c
c     second index, for one ii and all jj within jj..jjend
      do 9 j1=jj,jjend
      s=0.0d0
      do 10 jd=1,nhdimj
10    s=s+cic(j1,jd)*h2(jd)
9     h3(j1-jj+1)=s
c
c     store it in h and ind
      do 11 j1=jj,jjend
      hij=h3(j1-jj+1)
      int=int+1
      if(abs(hij).gt.tol)then
       inz=inz+1
       jt=jt+1
       ind(jt)=j1
       h(jt)=hij
      endif
11    continue
c
      jj=jjend
      goto 102
888   write(20) jt,(h(j1),j1=1,jt),(ind(j1),j1=1,jt)
      goto 101
c
999   write(6,6009)dble(inz)/real(int)*100.0d0,inz
6009  format(f10.4,'% of non-zero elements (',i10,')')
      close(20)
      write(6,*)fn(io),' finished'
10000 continue
c
      return
      end
c     ================================================================
      subroutine dhsdp(hsd,is,js,ns,O)
c     <is|H|js>, between two Slater dets
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      parameter (nort=700,nh110=2*nort)
      integer*4 is(nh110),js(nh110)
      dimension O(nort,nort)
      integer*4 slateri(nh110),slaterj(nh110),i
c
      hsd=0.0d0
      idif=0
c
c     determine how many orbitals are different and made
c     Slater determinant for each configuration
      is1=0
      js1=0
      do 1 i=1,ns
      if(is(i).ne.0)then
       is1=is1+1
       slateri(is1)=i
      endif
      if(js(i).ne.0)then
       js1=js1+1
       slaterj(js1)=i
      endif
      if(is(i).ne.js(i))then
       idif=idif+1
       if(idif.gt.2)return
      endif
1     continue
c
      if(js1.ne.is1)call report('is1<>js1')
c
c     determine sign of the element from Slater determinants
      kd=0
      sign=1.0d0
      ie=0
6     ie=ie+1
      kk=slateri(ie)
      je=0
4     je=je+1
      if(kk.eq.slaterj(je))then
c      put same orbital in j on the same place
       if(je.ne.ie)then
        sign=-sign
        it=slaterj(ie)
        slaterj(ie)=slaterj(je)
        slaterj(je)=it
       endif
       if(ie.lt.is1)goto 6
       goto 61
      else
       if(je.lt.js1)goto 4
      endif
c     this orbital k is not in j, put it in k
      kd=kd+1
      k=kk
      if(ie.lt.is1)goto 6
c
61    jd=0
      do 5 ie=1,is1
      if(slateri(ie).ne.slaterj(ie))then
       jd=jd+1
       j=slaterj(ie)
      endif
5     continue
      if(jd.ne.kd)call report('jd<>kd')
c
      if(jd.eq.0)then
c      same determinants (jd=0)
       do 2 i=1,is1
       k1=(slateri(i)+1)/2
2      hsd=hsd+O(k1,k1)
      else
       if(is(k).ne.js(j))call report('change of spin not allowed')
       k1=(k+1)/2
       j1=(j+1)/2
       hsd=O(k1,j1)
      endif
      hsd=hsd*sign
      return
      end
